*** WeiDU-185/src/biff.ml	Mon Feb 16 20:15:14 2004
--- WeiDU/src/biff.ml	Sat Feb  9 22:25:50 2008
***************
*** 1,6 ****
--- 1,15 ----
+ (* Note added due to LGPL terms.
+ 
+ This file was edited by Valerio Bigiani, AKA The Bigg, starting from
+ 6 November 2005. All changes for this file are listed in
+ diffs/src.biff.ml.diff file, as the output of a diff -Bw -c -N command.
+ 
+ It was originally taken from Westley Weimer's WeiDU 185. *)
+ 
  (* Infinity Engine [BIF] *)
  open Util
  open Key
+ open Cbif
  
  type biff_file = {
    res_loc       : int ;
***************
*** 29,41 ****
   * save the biff to the disk
   * returns an updated KEY _that can only be saved, cannot be used_ 
   *)
! let save_biff key filename components = begin
    let all_components = List.flatten (List.map (fun file ->
      let size = file_size file in
      if size > 0 then begin
!       let a,b = split (Filename.basename file) in
        try 
!         let tau = key_of_ext b in 
          [ (size,file,String.uppercase a,String.uppercase b,tau) ]
        with _ -> 
          log_only "WARNING: Not including [%s]: unknown resource type\n" 
--- 38,51 ----
   * save the biff to the disk
   * returns an updated KEY _that can only be saved, cannot be used_
   *)
! let save_biff key filename components = 
!   begin
    let all_components = List.flatten (List.map (fun file ->
      let size = file_size file in
      if size > 0 then begin
!       let a,b = split (Case_ins.filename_basename file) in
        try
!         let tau = key_of_ext false b in
          [ (size,file,String.uppercase a,String.uppercase b,tau) ]
        with _ ->
          log_only "WARNING: Not including [%s]: unknown resource type\n"
***************
*** 44,50 ****
    ) components) in
    let total_size = List.fold_left (fun acc (s,f,a,b,t) -> acc + s) 0
      all_components in
!   let tis_key = key_of_ext "TIS" in 
    let files,tiles = List.partition (fun (s,f,a,b,t) -> t <> tis_key)
     all_components in  
    let files = Array.of_list files in
--- 54,60 ----
    ) components) in
    let total_size = List.fold_left (fun acc (s,f,a,b,t) -> acc + s) 0
      all_components in
!   let tis_key = key_of_ext true "TIS" in
    let files,tiles = List.partition (fun (s,f,a,b,t) -> t <> tis_key)
     all_components in
    let files = Array.of_list files in
***************
*** 81,86 ****
--- 91,101 ----
      offset_data := !offset_data + s ;
    ) files ;
    Array.iteri (fun i (s,f,a,b,t) ->
+     let in_fd = Case_ins.unix_openfile f [Unix.O_RDONLY] 0 in
+     let istis = String.create 8 in
+       my_read 8 in_fd istis f ;
+     Unix.close in_fd ;
+     let s = (if istis = "TIS V1  " then s - 24 else s) in
      let off = offset_tiles + (i * 20) in
      let tis_loc = (i + 1) lsl 14 in
      write_int buff (off+0) tis_loc ; (* resource location *)
***************
*** 93,99 ****
      write_short buff (off+16) tis_type ;
      offset_data := !offset_data + s;
    ) tiles ; 
!   let out_fd = Unix.openfile filename [Unix.O_RDWR ; Unix.O_CREAT] 0 in 
    my_write buff_size out_fd buff filename ;
  
    let chunk_size = 10240 in
--- 108,114 ----
      write_short buff (off+16) tis_type ;
      offset_data := !offset_data + s;
    ) tiles ;
!   let out_fd = Unix.descr_of_out_channel (open_for_writing filename true) in
    my_write buff_size out_fd buff filename ;
  
    let chunk_size = 10240 in
***************
*** 110,127 ****
    in 
    Array.iteri (fun i (s,f,a,b,t) ->
      log_only "[%s] incorporating [%s]\n" filename f ;
!     let in_fd = Unix.openfile f [Unix.O_RDONLY] 0 in 
      copy_over in_fd f s ;
      Unix.close in_fd ;
    ) files ; 
    Array.iteri (fun i (s,f,a,b,t) ->
      log_only "[%s] incorporating [%s]\n" filename f ;
!     let in_fd = Unix.openfile f [Unix.O_RDONLY] 0 in 
      copy_over in_fd f s ;
      Unix.close in_fd ;
    ) tiles ; 
  
    Unix.close out_fd ;
  
    let total_size = file_size filename in 
  
--- 125,159 ----
    in
    Array.iteri (fun i (s,f,a,b,t) ->
      log_only "[%s] incorporating [%s]\n" filename f ;
!     let in_fd = Case_ins.unix_openfile f [Unix.O_RDONLY] 0 in
      copy_over in_fd f s ;
      Unix.close in_fd ;
    ) files ;
    Array.iteri (fun i (s,f,a,b,t) ->
      log_only "[%s] incorporating [%s]\n" filename f ;
!     let in_fd = Case_ins.unix_openfile f [Unix.O_RDONLY] 0 in
!     let istis = String.create 8 in
!       my_read 8 in_fd istis f ;
!     Unix.close in_fd ;
!     let in_fd = Case_ins.unix_openfile f [Unix.O_RDONLY] 0 in
!       if istis = "TIS V1  " then begin
!         let istis = String.create 24 in
!         (* have it skip the first 24 bytes *)
!         my_read 24 in_fd istis f ;
!         copy_over in_fd f (s - 24) ;
!       end else
          copy_over in_fd f s ;
+       ;
      Unix.close in_fd ;
    ) tiles ;
  
    Unix.close out_fd ;
+   if true then
+   begin
+     try
+       Case_ins.unix_chmod filename 511 ;
+     with e -> ()
+   end ;
  
    let total_size = file_size filename in
  
***************
*** 174,183 ****
    } 
  end 
  
- (* comment this out if you don't have zlib *)
- external uncompress : string -> pos:int -> clen:int -> ulen: int -> string
-   = "mlgz_uncompress"
- 
  (* reads 'size' bytes that would start at location 'start' in this BIFF
   * if it were not compressed! *)
  let read_compressed_biff_internal fd filename start size chunk_fun =
--- 206,211 ----
***************
*** 185,191 ****
    let unc_offset = ref 0 in 
  
    (* buffer holds the uncompressed bytes [start_unc,end_unc]  *)
!   let result = Buffer.create size in 
    let start_unc_offset = ref 0 in 
    let end_unc_offset = ref 0 in 
  
--- 213,219 ----
    let unc_offset = ref 0 in 
  
    (* buffer holds the uncompressed bytes [start_unc,end_unc]  *)
!   let (*result*) _ = Buffer.create size in
    let start_unc_offset = ref 0 in 
    let end_unc_offset = ref 0 in
  
***************
*** 218,224 ****
        let _ = Unix.lseek fd (!cmp_offset+8) Unix.SEEK_SET in 
        let cmp_buff = String.create cmplen in 
        my_read cmplen fd cmp_buff filename ;
!       let uncmp = uncompress cmp_buff 0 cmplen uncmplen in
        (*
        if (String.length uncmp <> uncmplen) then begin
          log_and_print "ERROR: [%s] chunk at offset %d was supposed to have %d bytes of compressed data that expanded to %d, but in reality they expanded to %d"
--- 246,252 ----
        let _ = Unix.lseek fd (!cmp_offset+8) Unix.SEEK_SET in 
        let cmp_buff = String.create cmplen in 
        my_read cmplen fd cmp_buff filename ;
!       let uncmp = Cbif.uncompress cmp_buff 0 cmplen uncmplen in
        (*
        if (String.length uncmp <> uncmplen) then begin
          log_and_print "ERROR: [%s] chunk at offset %d was supposed to have %d bytes of compressed data that expanded to %d, but in reality they expanded to %d"
***************
*** 342,350 ****
  
  let load_biff filename = 
    try 
!     let stats = Unix.stat filename in
      let size = stats.Unix.st_size in 
!     let fd = Unix.openfile filename [Unix.O_RDONLY] 0 in 
      let buff = String.create 20 in 
      let _ = Unix.read fd buff 0 20 in 
      if String.length buff < 8 then begin 
--- 370,378 ----
  
  let load_biff filename = 
    try 
!     let stats = Case_ins.unix_stat filename in
      let size = stats.Unix.st_size in 
!     let fd = Case_ins.unix_openfile filename [Unix.O_RDONLY] 0 in 
      let buff = String.create 20 in 
      let _ = Unix.read fd buff 0 20 in 
      if String.length buff < 8 then begin 
***************
*** 362,385 ****
        (Printexc.to_string e);
      raise e
  
! let check_file biff i =
    if i < 0 || i >= Array.length biff.files then begin
!     log_and_print "ERROR: BIFF [%s] has file entries 0--%d, cannot extract file at entry %d (this BIFF and your KEY file don't match)\n" biff.filename 
        ((Array.length biff.files) - 1) i ;
      failwith "invalid biff file entry" 
    end 
  
! let check_tile biff i =
    if i < 0 || i >= Array.length biff.files then begin
!     log_and_print "ERROR: BIFF [%s] has tileset entries 0--%d, cannot extract tileset at entry %d (this BIFF and your KEY file don't match)\n" biff.filename 
        ((Array.length biff.tilesets) - 1) i ;
      failwith "invalid biff tileset entry" 
    end 
  
! let extract_file biff i =
!   check_file biff i; 
!   let this = biff.files.(i) in
    try 
      let size = this.res_size in 
      if (biff.compressed) then begin
        read_compressed_biff biff.fd biff.filename this.res_offset size 
--- 390,413 ----
        (Printexc.to_string e);
      raise e
  
! let check_file biff i ign =
    if i < 0 || i >= Array.length biff.files then begin
!     if not ign then log_and_print "ERROR: BIFF [%s] has file entries 0--%d, cannot extract file at entry %d (this BIFF and your KEY file don't match)\n" biff.filename
        ((Array.length biff.files) - 1) i ;
      failwith "invalid biff file entry"
    end
  
! let check_tile biff i ign =
    if i < 0 || i >= Array.length biff.files then begin
!     if not ign then log_and_print "ERROR: BIFF [%s] has tileset entries 0--%d, cannot extract tileset at entry %d (this BIFF and your KEY file don't match)\n" biff.filename
        ((Array.length biff.tilesets) - 1) i ;
      failwith "invalid biff tileset entry"
    end
  
! let extract_file biff i ign =
    try
+ 	  check_file biff i ign;
+ 	  let this = biff.files.(i) in
      let size = this.res_size in
      if (biff.compressed) then begin
        read_compressed_biff biff.fd biff.filename this.res_offset size
***************
*** 390,405 ****
        buff 
      end 
    with e ->
!     log_and_print "ERROR: BIFF [%s]: unable to extract file %d\n" 
        biff.filename i ;
      raise e
  
! 
! let extract_tis biff i =
!   check_tile biff i; 
!   let this = biff.tilesets.(i) in
    try 
      let size = this.tis_number_of_tiles * this.tis_size_of_one_tile in 
      if (biff.compressed) then begin
        read_compressed_biff biff.fd biff.filename this.tis_offset size 
      end else begin 
--- 418,434 ----
        buff
      end
    with e ->
!     if not ign then log_and_print "ERROR: BIFF [%s]: unable to extract file %d\n"
        biff.filename i ;
      raise e
+ ;;
  
! let extract_tis biff i ign =
    try
+ 	  check_tile biff i ign;
+ 	  let this = biff.tilesets.(i) in
      let size = this.tis_number_of_tiles * this.tis_size_of_one_tile in
+     let buff =
  			if (biff.compressed) then begin
  	      read_compressed_biff biff.fd biff.filename this.tis_offset size
  	    end else begin
***************
*** 408,426 ****
        my_read size biff.fd buff biff.filename ;
        buff 
      end 
    with e ->
!     log_and_print "ERROR: BIFF [%s]: unable to extract tileset %d\n" 
        biff.filename i ;
      raise e
  
  let copy_file biff i oc is_tis =
    let size,offset = if is_tis then begin
!       check_tile biff i ; 
        let this = biff.tilesets.(i) in
        (this.tis_number_of_tiles * this.tis_size_of_one_tile) ,
        this.tis_offset
      end else begin
!       check_file biff i; 
        let this = biff.files.(i) in
        (this.res_size),
        this.res_offset
--- 437,468 ----
  	      my_read size biff.fd buff biff.filename;
  	      buff
  	    end
+ 	  in
+     let header = String.create 0x18 in
+     String.blit "TIS V1  " 0 header 0 8;
+     let str = str_of_int this.tis_number_of_tiles in
+     String.blit str 0 header 0x08 4 ;
+     let str = str_of_int this.tis_size_of_one_tile in
+     String.blit str 0 header 0x0c 4 ;
+     let str = str_of_int 0x18 in
+     String.blit str 0 header 0x10 4 ;
+     let str = str_of_int 64 in
+     String.blit str 0 header 0x14 4 ;
+     header ^ buff
    with e ->
!     if not ign then log_and_print "ERROR: BIFF [%s]: unable to extract tileset %d\n"
        biff.filename i ;
      raise e
+ ;;
  
  let copy_file biff i oc is_tis =
    let size,offset = if is_tis then begin
!       check_tile biff i false;
        let this = biff.tilesets.(i) in
        (this.tis_number_of_tiles * this.tis_size_of_one_tile) ,
        this.tis_offset
      end else begin
!       check_file biff i false; 
        let this = biff.files.(i) in
        (this.res_size),
        this.res_offset
